import pandas as pd


def set_prenome_column(df: pd.DataFrame) -> pd.DataFrame:
    try:
        df['PRENOME'] = df['NOME'].apply(lambda x: x.split()[0])
        return df
    except KeyError:
        raise KeyError('A chave NOME é obrigatória na planilha.')


def date_formatter(df: pd.DataFrame, birth_date_column: str) -> pd.DataFrame:
    df[f'{birth_date_column}_FORMATADA'] = pd.to_datetime(
        df[birth_date_column], format='"%Y-%m-%dT%H:%M:%SZ"'
    )

    df[f'{birth_date_column}_FORMATADA'] = df[f'{birth_date_column}_FORMATADA'].astype(str)

    return df



import json
from typing import List

import pandas as pd
from movva_tools.organization_service import \
    OrganizationService
from movva_tools.user_service import UserService
from sqlalchemy import desc, update

from movva_tools.base_service import BaseService
from movva_tools.constants import MAX_NAME_LEN, Flow, FlowType, GoFlowTypes
from movva_tools.decorators import ensure_transaction
from movva_tools.exceptions import (
    ObjectDoesNotExistException,
    ObjectDoesNotUpdatedExeption
)
from movva_tools.flow_models import RapidProFlows, RapidProFlowsRevision
from movva_tools.organization_models import RapidProOrganization
from movva_tools.validators import FlowNameValidator


class FlowService(BaseService):

    def __init__(self, db_connection=None) -> None:

        super().__init__(db_connection=db_connection)

        # model table entities
        self.flow = RapidProFlows
        self.flow_revision = RapidProFlowsRevision
        self.org = RapidProOrganization

        # services
        self.user_service = UserService(
            db_connection=self.db_connection
        )
        self.organization_service = OrganizationService(
            db_connection=self.db_connection
        )

    def __set_default_flow_metadata(self):

        return {
            "results": [],
            "dependencies": [],
            "waiting_exit_uuids": [],
            "parent_refs": []
        }

    def __set_initial_flow_revision_definition(self, flow):
        return {
            Flow.DEFINITION_NAME: flow.name,
            Flow.DEFINITION_UUID: str(flow.uuid),
            Flow.DEFINITION_SPEC_VERSION: Flow.CURRENT_SPEC_VERSION,
            Flow.DEFINITION_LANGUAGE: Flow.BASE_LANGUAGE,
            Flow.DEFINITION_TYPE: GoFlowTypes.TYPE_MESSAGE,
            Flow.DEFINITION_NODES: [],
            Flow.DEFINITION_UI: {},
            Flow.DEFINITION_REVISION: 1,
            Flow.DEFINITION_EXPIRE_AFTER_MINUTES: Flow.EXPIRES_AFTER_MINUTES
        }

    def __set_flow_revision_definition(self, flow, setted_definition):

        setted_definition[Flow.DEFINITION_NAME] = flow.name
        setted_definition[Flow.DEFINITION_REVISION] = 1
        setted_definition[Flow.DEFINITION_UUID] = str(flow.uuid)

        return setted_definition

    def is_valid_name(cls, name: str) -> bool:
        try:
            FlowNameValidator()(value=name)
            return True
        except Exception:
            return False

    def get_flow(self, flow_name):

        flow = self.db_connection.session.query(self.flow).filter_by(
            name=flow_name
        ).first()

        if flow:
            return flow
        else:
            raise ObjectDoesNotExistException(dababase_object=self.flow)

    def create_new_flow_revision(self, flow, user, setted_definition=None):
        if setted_definition:
            definition = self.__set_flow_revision_definition(
                flow, setted_definition=setted_definition
            )
        else:
            definition = self.__set_initial_flow_revision_definition(flow=flow)

        new_flow_revision = self.flow_revision(
            created_by_id=user.id,
            flow_id=flow.id,
            definition=definition
        )

        self.add(new_flow_revision)
        self.flush()
        return new_flow_revision

    def _create_flow(
        self,
        org_name,
        flow_name,
        flow_type=FlowType.TYPE_MESSAGE,
        expires_after_minutes=Flow.EXPIRES_AFTER_MINUTES,
        create_revision: bool = False,
        external_context: bool = False
    ):
        new_flow_revision = None
        user = self.user_service.get_default_user()
        org = self.organization_service.get_org(org_name)

        new_flow = self.flow(
            org_id=org.id,
            created_by_id=user.id,
            name=flow_name,
            flow_type=flow_type,
            expires_after_minutes=expires_after_minutes,
            flow_metadata=self.__set_default_flow_metadata()
        )

        self.add(new_flow)
        self.flush()

        if create_revision:
            new_flow_revision = self.create_new_flow_revision(
                flow=new_flow,
                user=user
            )

        return new_flow, new_flow_revision

    def create(
        self,
        org_name,
        flow_name,
        flow_type=FlowType.TYPE_MESSAGE,
        expires_after_minutes=Flow.EXPIRES_AFTER_MINUTES,
        create_revision: bool = False,
        commit_after_create: bool = False,
        external_context: bool = True
    ):

        self.is_valid_name(flow_name)

        if not external_context:

            with self.db_connection.session.begin():
                new_flow, new_flow_revision = self._create_flow(
                    org_name=org_name,
                    flow_name=flow_name,
                    flow_type=flow_type,
                    expires_after_minutes=expires_after_minutes,
                    create_revision=create_revision,
                    external_context=external_context
                )
        else:
            new_flow, new_flow_revision = self._create_flow(
                org_name=org_name,
                flow_name=flow_name,
                flow_type=flow_type,
                expires_after_minutes=expires_after_minutes,
                create_revision=create_revision,
                external_context=external_context
            )

        if commit_after_create:
            # Commit a transação após os blocos 'with'
            self.commit()

        return new_flow, new_flow_revision


class FlowCopyService(BaseService):

    def __init__(self, db_connection=None) -> None:

        super().__init__(db_connection)

        # services
        self.flow_service = FlowService(
            db_connection=self.db_connection
        )
        self.organization_service = OrganizationService(
            db_connection=self.db_connection
        )
        self.user_service = UserService(
            db_connection=self.db_connection
        )

        # model table entities
        self.flow_revision = RapidProFlowsRevision

    def get_unique_name(self, org, base_name):

        name = f'{base_name[:MAX_NAME_LEN]}' if len(base_name) > MAX_NAME_LEN else base_name
        qs = self.db_connection.session.query(self.flow).filter_by(
            org_id=org.id,
            is_active=True,
            name=name
        ).all()

        if qs:
            name = 'Copy_of ' + name

        return name

    def get_last_flow_revision(self, flow):

        flow_revision = self.db_connection.session.query(
            self.flow_revision
        ).order_by(
            desc(self.flow_revision.created_on)
        ).filter_by(
            flow_id=flow.id
        ).first()

        if flow_revision:
            return flow_revision
        else:
            raise ObjectDoesNotExistException(
                dababase_object=self.flow_revision
            )

    def transfer_flow_revision_messages(self, json_string: str, df: pd.DataFrame):
        messages = df.loc[:, ['MENSAGENS ORIGINAIS', 'MENSAGENS NOVAS']]
        messages = messages.dropna(how='all')

        if messages.empty:
            raise Exception('Without Messages.')

        for _, actual, new in messages.itertuples():
            actual = actual.replace('\n', '\\n')
            new = new.replace('\n', '\\n')
            actual = f'"{actual}"'
            new = f'"{new}"'
            json_string = json_string.replace(actual, new)

        return json_string

    def transfer_flow_revision_data(self, flow_json: dict, spreadsheet_data: pd.DataFrame):
        flow_data = json.dumps(flow_json, ensure_ascii=False)

        flow_data = self.transfer_flow_revision_messages(
            json_string=flow_data,
            df=spreadsheet_data
        )
        return json.loads(flow_data)

    def migrate_definition(
        self, flow_json, flow, flow_revision_destination, user,
        data: pd.DataFrame = None
    ):
        flow_revision_copy = flow_json.copy()

        # aqui deve entrar as modificações do flow revision
        if not data.empty:
            flow_revision_copy = self.transfer_flow_revision_data(
                spreadsheet_data=data,
                flow_json=flow_revision_copy
            )

        if flow_revision_destination:
            definition_destination = json.loads(
                flow_revision_destination.definition
            )
            flow_revision_copy['name'] = definition_destination['name']

            flow_revision_copy_json = json.dumps(flow_revision_copy)

            query = update(
                self.flow_revision
            ).where(
                self.flow_revision.id == flow_revision_destination.id
            ).values(
                definition=flow_revision_copy_json
            ).returning(self.flow_revision)

            if result := self.db_connection.execute_query(query).fetchone():
                return result
            else:
                raise ObjectDoesNotUpdatedExeption(
                    dababase_object=self.flow_revision
                )
        else:
            flow_revision_destination = self.flow_service.create_new_flow_revision(
                flow=flow, user=user,
                setted_definition=flow_revision_copy
            )

        return flow_revision_destination.definition

    def import_definition(
        self,
        user, definition,
        flow_destination, flow_revision_destination, org_destination,
        data: pd.DataFrame = None
    ):
        """
            Allows setting the definition for a flow from another definition.
            All UUID's will be remapped.
        """

        flow_revision_migrated_data = self.migrate_definition(
            flow_json=definition,
            flow=flow_destination,
            user=user,
            flow_revision_destination=flow_revision_destination,
            data=data
        )

        flow_revision_migrated = self.get_last_flow_revision(
            flow=flow_destination
        )

        return flow_revision_migrated

    def get_definition(self, flow) -> dict:
        revision = self.get_last_flow_revision(
            flow=flow
        )

        definition = revision.definition

        return json.loads(definition)

    @ensure_transaction
    def clone(self, base_flow_name, flow_suggested_name, destiny_organization_name, data: List[List[str]] = None):
        """
        Returns a clone of this flow
        """
        if data:
            data = pd.DataFrame(data[1:], columns=data[0])

        with self.db_connection.session.begin():
            user = self.user_service.get_default_user()
            org = self.organization_service.get_org(
                org_name=destiny_organization_name
            )

            copy, revision = self.flow_service.create(
                org_name=org.name,
                flow_name=flow_suggested_name
            )

            base_flow = self.flow_service.get_flow(
                flow_name=base_flow_name
            )

            base_flow_json = self.get_definition(
                flow=base_flow
            )

            self.import_definition(
                user=user,
                definition=base_flow_json,
                flow_destination=copy,
                flow_revision_destination=revision,
                org_destination=org,
                data=data
            )



import json
import uuid
from datetime import datetime

from sqlalchemy import (
    JSON, UUID, Boolean, Column, DateTime,
    Integer, String
)

from movva_tools.databases import Base


class RapidProFlows(Base):
    __tablename__ = 'flows_flow'

    id = Column(Integer, primary_key=True)
    uuid = Column(UUID(as_uuid=True), default=uuid.uuid4)
    name = Column(String)
    is_active = Column(Boolean)
    is_system = Column(Boolean)
    created_by_id = Column(Integer)
    modified_by_id = Column(Integer)
    saved_by_id = Column(Integer)
    saved_on = Column(DateTime, default=datetime.utcnow)
    created_on = Column(DateTime, default=datetime.utcnow)
    modified_on = Column(DateTime, default=datetime.utcnow)
    flow_type = Column(String)
    flow_metadata = Column('metadata', JSON, default={"results": [], "dependencies": [], "waiting_exit_uuids": [], "parent_refs": []})
    is_archived = Column(Boolean)
    expires_after_minutes = Column(Integer)
    base_language = Column(String)
    org_id = Column(Integer)
    ignore_triggers = Column(Boolean)
    has_issues = Column(Boolean)
    version_number = Column(String)

    def __init__(
        self, created_by_id, expires_after_minutes, org_id, name,
        flow_metadata, flow_type
    ):
        self.name = name
        self.created_by_id = created_by_id
        self.saved_by_id = created_by_id
        self.modified_by_id = created_by_id
        self.expires_after_minutes = expires_after_minutes
        self.org_id = org_id
        self.flow_metadata = flow_metadata
        self.base_language = 'base'
        self.is_active = True  # Set default value
        self.flow_type = flow_type
        self.is_archived = False  # Set default value
        self.is_system = False
        self.ignore_triggers = False
        self.has_issues = False
        self.version_number = '13.1.0'

    def __repr__(self):
        return f'id:{self.id} <{self.name}>'


class RapidProFlowsRevision(Base):
    __tablename__ = 'flows_flowrevision'

    id = Column(Integer, primary_key=True)
    is_active = Column(Boolean)
    created_on = Column(DateTime, default=datetime.utcnow)
    modified_on = Column(DateTime, default=datetime.utcnow)
    definition = Column(JSON, default={"name": "Fluxo Interface", "uuid": "flow id", "spec_version": "13.1.0", "language": "base", "type": "messaging", "nodes": [], "_ui": {}, "revision": 1, "expire_after_minutes": 10080})
    spec_version = Column(String)
    revision = Column(Integer)
    created_by_id = Column(Integer)
    flow_id = Column(Integer)
    modified_by_id = Column(Integer)

    # constants

    MSG_TYPE = 'send_message'
    WAITING_RESPONSE_TYPE = 'switch'
    OPERAND = '@input.text'

    def __init__(
        self, created_by_id, flow_id,
        definition
    ):
        self.created_by_id = created_by_id
        self.modified_by_id = created_by_id
        self.flow_id = flow_id
        self.definition = definition
        self.revision = 1
        self.spec_version = '13.1.0'
        self.is_active = True  # Set default value

    def __repr__(self):
        flow_name = json.loads(self.definition)["name"]
        return f'id:{self.id} <{flow_name}> Revision:{self.revision}'

    @staticmethod
    def filter_node(node_uuid: str, definition: list) -> tuple:
        for idx, uid in enumerate(definition['nodes']):
            if uid['uuid'] == node_uuid:
                return (uid, idx)
        return ([], None)

    @staticmethod
    def search_node_by_pos(node_pos, definition: list) -> tuple:
        for idx, uid in enumerate(definition['nodes']):
            if idx == node_pos:
                return (uid, idx)
        return ([], None)

    def set_exits(
            self, node_uuid_to=None, node_uuid_for_destination=None
    ) -> list:

        exits_uuid = str(uuid.uuid4())

        exits_initial = [{}]

        exits_initial[0].update(
            {
                "destination_uuid": node_uuid_for_destination,
                'uuid': exits_uuid
            }
        )

        if node_uuid_to:
            node, position = self.filter_node(
                node_uuid=node_uuid_to, definition=self.definition
            )

            if not node:
                raise Exception('Node não encontrado.')

            if node.get('exits', None):
                self.definition['nodes'][position] = exits_initial
            else:
                self.definition['nodes'][position]['exits'].append(exits_initial)
        else:
            self.definition['nodes'][-1].update(
                {'exits': exits_initial}
            )

        return self.definition

    def set_action(self, text: str, type: str, quick_replies=[], attachments=[]) -> dict:
        action_dict = {}

        action_uuid = str(uuid.uuid4())

        action_dict.update(
            {
                'type': type,
                'text': text,
                'quick_replies': quick_replies,
                'uuid': action_uuid,
                'attachments': attachments
            }
        )

        return action_dict

    def set_send_message_node(self, text: str) -> dict:

        node_uuid = str(uuid.uuid4())

        action_dict = self.set_action(text=text, type=self.MSG_TYPE)

        self.definition['nodes'].append(
            {}
        )

        self.definition['nodes'][0].update(
            {
                'uuid': node_uuid
            }
        )
        self.definition['nodes'][0]['actions'] = []

        self.definition['nodes'][0]['actions'].append(action_dict)

        return self.definition

    def set_categories_wait_for_response_node(self, categories: list, definition: list):

        # categories must be [{name: nome, exit_node_pos: 1}]

        # node, position = self.filter_node(
        #     node_uuid=node_uuid,
        #     definition=definition
        # )

        categories = []

        for category in categories:

            node_exit, position_exit = self.search_node_by_pos(
                node_pos=category['exit_node_pos'],
                definition=definition
            )

            categories.append(
                {
                    'uuid': str(uuid.uuid4()),
                    'name': category['name'],
                    'exit_uuid': node_exit['uuid'],
                    'position_node_exit': position_exit
                }
            )

        return categories

    def set_cases_waiting_for_response_node(
        self, types: list, arguments: list
    ) -> list:

        # types and arguments format must be list of strings
        # arguments can be list of lists

        if len(types) != len(arguments):
            raise Exception('Incompatible')

        cases = []
        for type_case, args in zip(types, arguments):
            cases.append(
                {
                    'uuid': str(uuid.uuid4()),
                    'type': type_case,
                    'arguments': args
                }
            )

        return cases

    def set_waiting_for_response_node(
        self, text: str, name: str, exit_uuid: str, definition: list, categories: list,
        result_name: str
    ) -> dict:

        # categories must be [{key1: value1}]

        uuid_node = str(uuid.uuid4())

        router = {
            'type': self.WAITING_RESPONSE_TYPE,
            'cases': [],
            'default_category_uuid': str(uuid.uuid4()),
            'categories': [{}],
            'operand': self.OPERAND,
            'wait': {
                'type': 'msg'
            },
            'result_name': result_name
        }

        router['categories'] = self.set_categories_wait_for_response_node(
            categories=categories,
            definition=definition
        )

        router['cases'] = []

        waiting_response_node = {
            'uuid': uuid_node,
            'router': router
        }

        definition['router'].append(waiting_response_node)

        return definition

    def create(self, flow_name, flow_uuid, user_id):

        return {
            "name": flow_name,
            "uuid": flow_uuid,
            "spec_version": "13.1.0",
            "language": "base",
            "type": "messaging",
            "nodes": [],
            "_ui": {},
            "revision": 1,
            "expire_after_minutes": 10080,
            "localization": {}
        }


class RapidProFlowChannelDependencies(Base):
    __tablename__ = 'flows_flow_channel_dependencies'

    id = Column(Integer, primary_key=True)
    flow_id = Column(Integer)
    channel_id = Column(Integer)


class RapidProFlowFieldDependencies(Base):
    __tablename__ = 'flows_flow_field_dependencies'

    id = Column(Integer, primary_key=True)
    flow_id = Column(Integer)
    contactfield_id = Column(Integer)


class RapidProFlowLabel(Base):
    __tablename__ = 'flows_flowlabel'

    id = Column(Integer, primary_key=True)
    uuid = Column(UUID(as_uuid=True), default=uuid.uuid4)
    name = Column(String)
    org_id = Column(Integer)
    parent_id = Column(Integer)



import uuid
from datetime import datetime

from sqlalchemy import (
    UUID, Boolean, Column, DateTime,
    Integer, String
)

from movva_tools.databases import Base
from movva_tools.constants import ContactField


class RapidProContactFields(Base):
    __tablename__ = "contacts_contactfield"

    id = Column(Integer, primary_key=True)
    is_active = Column(Boolean)
    created_on = Column(DateTime, default=datetime.utcnow)
    modified_on = Column(DateTime, default=datetime.utcnow)
    uuid = Column(UUID(as_uuid=True), default=uuid.uuid4)
    label = Column(String)
    key = Column(String)
    value_type = Column(String)
    show_in_table = Column(Boolean)
    priority = Column(Integer)
    field_type = Column(String)
    created_by_id = Column(Integer)
    modified_by_id = Column(Integer)
    org_id = Column(Integer)

    def __init__(
        self, created_by_id, org_id,
        key, label, field_type, priority=None
    ):
        self.value_type = ContactField.TYPE_TEXT
        self.show_in_table = False
        self.priority = 0 if not priority else priority

        self.created_by_id = created_by_id
        self.modified_by_id = created_by_id

        self.org_id = org_id
        self.key = key
        self.label = label
        self.field_type = field_type
        self.is_active = True  # Set default value

    def __repr__(self):
        return f'<{self.uuid}> key:{self.key} label:{self.label} org_id:{self.org_id}'



from movva_tools.organization_models import RapidProOrganization
from movva_tools.exceptions import ObjectDoesNotExistException
from movva_tools.base_service import BaseService


class OrganizationService(BaseService):

    def __init__(self, db_connection=None) -> None:
        super().__init__(db_connection)

        self.org = RapidProOrganization

    def get_org(self, org_name):

        org = self.db_connection.session.query(self.org).filter_by(
            name=org_name
        ).first()

        if org:
            return org
        else:
            raise ObjectDoesNotExistException(dababase_object=self.org)

    def get_token_org(self, db_connection=None):
        pass



from movva_tools.constants import MAX_NAME_LEN
from movva_tools.exceptions import (
    FlowNameLongerException,
    FlowNameInvalidCharacterException,
    FlowNameContainsNullCharacterException,
    FlowNameStartsOrEndWithWhitespaceException
)


class FlowNameValidator:
    """
    Validator for names of flows and their dependencies.
    """

    def __init__(self, max_length=MAX_NAME_LEN):
        self.max_length = max_length

    def __call__(self, value):
        # model forms will add their own validator based on max_length but we need this for validating for imports etc
        if len(value) > self.max_length:
            raise FlowNameLongerException(self.max_length)

        if value != value.strip():
            raise FlowNameStartsOrEndWithWhitespaceException()

        for ch in '"\\':
            if ch in value:
                raise FlowNameInvalidCharacterException(ch)

        if "\0" in value:
            raise FlowNameContainsNullCharacterException()



from movva_tools.constants import DEFAULT_USER_ID
from movva_tools.user_models import RapidProUser
from movva_tools.exceptions import ObjectDoesNotExistException
from movva_tools.base_service import BaseService


class UserService(BaseService):

    def __init__(self, db_connection=None) -> None:
        super().__init__(db_connection=db_connection)

        # model table entities
        self.user = RapidProUser

    def get_default_user(self):

        default_user = self.db_connection.session.query(self.user).filter_by(
            id=DEFAULT_USER_ID
        ).first()

        if default_user:
            return default_user
        else:
            raise ObjectDoesNotExistException(dababase_object=self.user)



import json
import os
import re

from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from google.cloud.secretmanager import SecretManagerServiceClient
from googleapiclient.discovery import build
from movva_tools.constants import GoogleSheets


# If modifying these scopes, delete the file token.json.
SCOPES = GoogleSheets.READ_ONLY_SCOPE


class GoogleSheetsIntegration:

    def __init__(self, link, spreadsheet_page):
        self.__link = link
        self.spreadsheet_page = spreadsheet_page
        self.service = None
        self._secret_manager_service = None
        self.__spreadsheet_id = None

    def check_data(self):
        """
            Verifica se as informações necessárias para a classe
            foram informadas.
        """
        if not self.__link or not self.__spreadsheet_id or not self.spreadsheet_page:
            raise Exception('The spreadsheet link, name and id must be informed')

    def __extract_id_from_google_sheets_link(self):
        """
            Extrai o id do link da planilha do Google através do
            padrão de expressão regular
        """

        pattern = r'/d/([a-zA-Z0-9-_]+)'

        # Procura pelo padrão no link fornecido
        match = re.search(pattern, self.__link)

        # Se houver uma correspondência, retorna o ID encontrado, caso contrário, retorna None
        if match:
            return match.group(1)
        else:
            raise Exception('Planilha não encontrada.')

    def __fetch_credentials_data(self):

        environment = None
        token_path = None
        smanager_creds = None
        creds_path = os.environ.get('SECRET_MANAGER_PATH', None)

        if not creds_path:
            token_path = os.environ.get('TOKEN_LOCAL_PATH', '')
            creds_path = os.environ.get('CREDENTIALS_LOCAL_PATH', None)
            environment = 'local'
        else:
            smanager_creds = os.environ.get('APPLICATION_DEFAULT_CREDENTIALS', None)
            environment = 'production'

        return creds_path, smanager_creds, token_path, environment

    def _validate_google_sheets_token(self, creds, gsheets_token_path, environment):
        if environment == 'local':
            # The file token.json stores the user's access and refresh tokens, and is
            # created automatically when the authorization flow completes for the first
            # time.

            # If there are no (valid) credentials available, let the user log in.
            if not creds or not creds.valid:
                if creds and creds.expired and creds.refresh_token:
                    creds.refresh(Request())
                else:
                    flow = InstalledAppFlow.from_client_secrets_file(
                        self.__creds_path, SCOPES
                    )
                    creds = flow.run_local_server(port=0)
                    # Save the credentials for the next run
                    print('Openning token.json file...')
                    with open(gsheets_token_path, 'w') as token:
                        token.write(creds.to_json())

        else:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())

        return creds

    def __authorize_google_sheets(self):
        """
            Cria/carrega credenciais de acesso da API do Google Sheets
            e inicia a conexão com a API.
        """
        creds_path, smanager_creds, token_path, environment = self.__fetch_credentials_data()

        if environment != 'local':
            sm_creds = json.loads(smanager_creds)
            smanager_credentials = Credentials.from_authorized_user_info(
                sm_creds,
                scopes=['https://www.googleapis.com/auth/cloud-platform']
            )

            self._secret_manager_service = SecretManagerServiceClient(
                credentials=smanager_credentials
            )
            response = self._secret_manager_service.access_secret_version(
                name=creds_path
            )
            secret_payload = response.payload.data.decode("UTF-8").split('\n')

            gsheets_token_path = json.loads(
                '{' + secret_payload[1].split('={')[-1]
            )

            creds = Credentials.from_authorized_user_info(gsheets_token_path, SCOPES)
        else:
            gsheets_token_path = token_path

            if os.path.exists(gsheets_token_path):
                creds = Credentials.from_authorized_user_file(
                    gsheets_token_path, SCOPES
                )

        creds = self._validate_google_sheets_token(creds, gsheets_token_path, environment)

        # Create Google Sheets connection
        service = build('sheets', 'v4', credentials=creds)

        return service

    def read_sheet_data(self, end_selection: str = None) -> list:
        """
            Lê e armazena em uma estrutura de listas o conteúdo da planilha.
        """

        self.__spreadsheet_id = self.__extract_id_from_google_sheets_link()

        self.check_data()

        self.service = self.__authorize_google_sheets()

        # Call the Sheets API
        end_selection = end_selection if end_selection else 'BZ'
        sheet = self.service.spreadsheets()
        result = sheet.values().get(
            spreadsheetId=self.__spreadsheet_id,
            range=f'{self.spreadsheet_page}!A1:{end_selection}'
        ).execute()

        values = result.get('values', [])

        if not values:
            print('No data found.')
            return

        return values



from movva_tools.databases import RapidProDatabase
from movva_tools.contacts_models import RapidProContactFields


class ContactService:

    def __init__(self, db_connection: RapidProDatabase) -> None:

        # model table entities
        self.contact_fields = RapidProContactFields
        self.db_connection = db_connection

    def get_or_create(self, org, flow, user, key: str, name: str = None, value_type=None):

        self.db_connection.set_session()
        changed = False
        with self.db_connection.session.begin():
            existing = self.contact_fields.session.query(
                self.contact_fields
            ).filter(
                is_active=True,
                key=key
            ).first()

            if existing:
                if name and existing.name != name:
                    # existing.name = name
                    changed = True

                # update our type if we were given one
                if value_type and existing.value_type != value_type:

                    # existing.value_type = value_type
                    changed = True

                if changed:
                    existing.modified_by = user.id
                    existing.update().where(
                        self.contact_fields.id == existing.id
                    ).values(name=name, value_type=value_type)

                return existing



from movva_tools.import_contacts import ImportContacts
from movva_tools.google_integration import GoogleSheetsIntegration
import pandas as pd
import asyncio
import time
import os


async def main():
    sheets_integration = GoogleSheetsIntegration(
        creds_path='movva_tools/credentials.json',
        spreadsheet_name='SPREADSHEET_NAME',
        link='LINK'
    )

    data = sheets_integration.read_sheet_data()

    dataframe = pd.DataFrame(data[1:], columns=data[0])

    rapidpro_integration = ImportContacts(
        url=os.environ.get('RAPIDPRO_URL'),
        organization_token=os.environ.get('ORGANIZATION_TOKEN')  # Encontrado no RapidPro
    )

    erase_contacts = False

    if erase_contacts:
        start_time = time.time()
        rapidpro_integration.set_client()
        rapidpro_integration.delete_contacts(group_name='imported 09/08/2023')
        end_time = time.time()
    else:
        start_time = time.time()
        await rapidpro_integration.import_contacts(df=dataframe)
        end_time = time.time()

    # Calcular o tempo total de execução
    execution_time = end_time - start_time

    if execution_time >= 60:
        # Caso o tempo de execução seja maior ou igual a 60 segundos,
        # faz a conversão para minutos e segundos
        minutes = int(execution_time // 60)
        seconds = execution_time % 60
        print(f"Tempo de execução: {minutes} minutos e {seconds:.4f} segundos")
    else:
        # Caso o tempo de execução seja menor que 60 segundos, mostra apenas em segundos
        print(f"Tempo de execução: {execution_time:.4f} segundos")

if __name__ == '__main__':
    asyncio.run(main())



from enum import Enum
import os


MAX_NAME_LEN = 64
DEFAULT_USER_ID = int(os.environ.get('DEFAULT_USER_ID'))
READ_ONLY_ISOLATION_LEVEL_CONNECTION = '?isolation_level=READ%20ONLY'


class BaseEnum(Enum):

    def __get__(self, instance, owner):
        return self.value


class GoogleSheets(BaseEnum):

    DEFAULT_SCOPE = ['https://www.googleapis.com/auth/spreadsheets']
    READ_ONLY_SCOPE = ['https://www.googleapis.com/auth/spreadsheets.readonly']


class ContactField(BaseEnum):
    MAX_KEY_LEN = 36
    MAX_NAME_LEN = 36

    TYPE_TEXT = "T"
    TYPE_NUMBER = "N"
    TYPE_DATETIME = "D"
    TYPE_STATE = "S"
    TYPE_DISTRICT = "I"
    TYPE_WARD = "W"

    TYPE_CHOICES = {
        TYPE_TEXT, "Text",
        TYPE_NUMBER, "Number",
        TYPE_DATETIME, "Date & Time",
        TYPE_STATE, "State",
        TYPE_DISTRICT, "District",
        TYPE_WARD, "Ward",
    }

    ENGINE_TYPES = {
        TYPE_TEXT: "text",
        TYPE_NUMBER: "number",
        TYPE_DATETIME: "datetime",
        TYPE_STATE: "state",
        TYPE_DISTRICT: "district",
        TYPE_WARD: "ward",
    }

    # fixed keys for system-fields
    KEY_ID = "id"
    KEY_NAME = "name"
    KEY_CREATED_ON = "created_on"
    KEY_LANGUAGE = "language"
    KEY_LAST_SEEN_ON = "last_seen_on"

    # fields that cannot be updated by user
    IMMUTABLE_FIELDS = (KEY_ID, KEY_CREATED_ON, KEY_LAST_SEEN_ON)

    SYSTEM_FIELDS = {
        KEY_ID: {"name": "ID", "value_type": TYPE_NUMBER},
        KEY_NAME: {"name": "Name", "value_type": TYPE_TEXT},
        KEY_CREATED_ON: {"name": "Created On", "value_type": TYPE_DATETIME},
        KEY_LANGUAGE: {"name": "Language", "value_type": TYPE_TEXT},
        KEY_LAST_SEEN_ON: {"name": "Last Seen On", "value_type": TYPE_DATETIME},
    }


class Flow(BaseEnum):
    BASE_LANGUAGE = "base"

    DEFINITION_UUID = "uuid"
    DEFINITION_NAME = "name"
    DEFINITION_SPEC_VERSION = "spec_version"
    DEFINITION_TYPE = "type"
    DEFINITION_LANGUAGE = "language"
    DEFINITION_REVISION = "revision"
    DEFINITION_EXPIRE_AFTER_MINUTES = "expire_after_minutes"
    DEFINITION_METADATA = "metadata"
    DEFINITION_NODES = "nodes"
    DEFINITION_UI = "_ui"

    INSPECT_DEPENDENCIES = "dependencies"

    EXPIRES_AFTER_MINUTES = 60 * 24 * 7  # 1 week

    CURRENT_SPEC_VERSION = "13.1.0"


class FlowType(BaseEnum):

    TYPE_MESSAGE = "M"
    TYPE_BACKGROUND = "B"
    TYPE_SURVEY = "S"
    TYPE_VOICE = "V"
    TYPE_USSD = "U"


class GoFlowTypes(BaseEnum):
    TYPE_MESSAGE = "messaging"
    TYPE_BACKGROUND = "messaging_background"
    TYPE_SURVEY = "messaging_offline"
    TYPE_VOICE = "voice"


class ExceptionMessages(BaseEnum):

    OBJECT_DOES_NOT_EXISTS = 'The object of class {} does not exists or had not been found.'
    OBJECT_DOES_NOT_UPDATED = 'The object of class {} does not updated.'

    FLOW_NAME_LONGER = 'Cannot be longer than {} characters.'
    FLOW_NAME_STARTS_OR_END_WITH_WHITESPACE = 'Cannot begin or end with whitespace.'
    FLOW_INVALID_CHARACTERS = 'Cannot contain the character: {}'
    FLOW_NAME_CONTAINS_NULL_CHARACTER = "Cannot contain null characters."
